/*==============================================================================

FICHIER     : [avancement.h]

DATE        : 2006/01/0005 21:13:17

CREATEUR    : [Linux!jef]

COMMENTAIRE :

================================================================================

==============================================================================*/

#ifndef __AVANCEMENT_H__
#define __AVANCEMENT_H__

#include <gnome.h>

/*@$#[avancement.c] global proto. AutoProtoSigV1.1. date: 2010/02/03 22:09:53 */
#include "proto.h"
#ifdef __cplusplus
extern "C" {
#endif
void PushAvancement(void);
void PopAvancement(void);
void InitAvancement(int sectors);
void CreateAvancement(GtkWidget *forme);
void HideMpegVisu(void);
void DestroyAvancement(void);
int Avancement(int lenReaded);
int AvancementAbs(long pos);
void SetTotalSize(long long tSize);
void SetTotalSteps(int steps);
void NewStep(void);
int AvancementUi(void);
int AvancementUi2(double pcts);
void DisplayAvancement(char *format, ...);
void TotalTime(char *string);
void ResetUserStop(void);
int IsUserStop(void);
#ifdef __cplusplus
}
#endif
/*@$% end of AutoProtoSigV1.1 (Dont remove this line) []*/
#endif	/* __AVANCEMENT_H__ */

